<?php

namespace App\Service;

use App\Exception;
use App\ParamsException;

class IntegrationService {

    private $util;
    private $consumerService;
    private $productService;
    private $branchFactory;
    private $orderFactory;
    private $paymentFactory;
    private $integrationFactory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Service\ProductService $productService, 
                                 \App\Factory\BranchFactory $branchFactory, 
                                 \App\Factory\OrderFactory $orderFactory, 
                                 \App\Factory\PaymentFactory $paymentFactory,
                                 \App\Factory\IntegrationFactory $integrationFactory) {

        $this->util               = $util;
        $this->consumerService    = $consumerService;
        $this->productService     = $productService;
        $this->branchFactory      = $branchFactory;
        $this->orderFactory       = $orderFactory;
        $this->paymentFactory     = $paymentFactory;
        $this->integrationFactory = $integrationFactory;
    }
    
    public function generalParameters($branch, $redirectStore, $merchantConfiguration) {

        $vendor  = $this->branchFactory->getStandardVendor($branch);
        $cashier = $this->branchFactory->getStandardCashier($branch, $vendor);

        if($redirectStore) {
            if(empty($merchantConfiguration['CDLOJA'])) {
                throw new Exception\ParamsException("Não existe um código de loja associado ao restaurante <b>{$merchantConfiguration['IDTOKEN']}</b> iFood.");
            }

            if(empty($merchantConfiguration['CDCAIXA'])) {
                throw new Exception\ParamsException("Não existe um código de caixa associado ao restaurante <b>{$merchantConfiguration['IDTOKEN']}</b> iFood.");
            }

            $store   = $merchantConfiguration['CDLOJA'];
            $cashier = [
                'CDCAIXA'    => $merchantConfiguration['CDCAIXA'],
                'CDOPERADOR' => $cashier['CDOPERADOR']
            ];
        } else {
            $store = $this->branchFactory->getStandardStore($branch, $vendor);
        }

        $idControlProd = $this->branchFactory->getProductControlId($branch, $store);
        $state         = $this->branchFactory->getBranchState($branch);
        
        return [
            'CDFILIAL'     => $branch,
            'CDVENDEDOR'   => $vendor,
            'CDLOJA'       => $store,
            'CDCAIXA'      => $cashier["CDCAIXA"],
            'CDOPERADOR'   => $cashier["CDOPERADOR"],
            'IDCONTROPROD' => $idControlProd,
            'SGESTADO'     => $state
        ];
    }
    
    public function validateCashierSinc($filial) {
        $caixaSinc = $this->orderFactory->fetchIsCashierSinc($filial);

        if(!isset($caixaSinc) || empty($caixaSinc)) {
            throw new Exception\ParamsException("Caixa sincronizador não encontrado na filial {$filial}, Habilite a flag de sincronização de caixa.");
        }
    }
    
    public function openOrderBill($generalParams, $order, $consumer, $deliveryRules) {
        if(empty($generalParams["CDOPERADOR"])) {
            throw new Exception\ParamsException("
                Não existe um operador padrão atrelado ao vendedor padrão {$generalParams['CDVENDEDOR']} da filial {$generalParams['CDFILIAL']}.
                Para atrelar o operador padrão ao vendedor padrão desta filial, verifique a seção 'Problemas Frequentes', no manual da integração.");
        } 
        
        $operator = $this->orderFactory->validateOperator($generalParams);
        if(empty($operator)) {
            throw new Exception\ParamsException("
                O operador {$generalParams['CDOPERADOR']} não está associado a filial {$generalParams['CDFILIAL']}.<br>
                Para atrelar o operador a esta filial, verifique a seção 'Problemas Frequentes', no manual da integração.");
        }
        
        $isDuplicated = $this->orderFactory->isDuplicated($generalParams, $order); //validateDuplication
        if($isDuplicated) {
            throw new Exception\IntegrationException("
                O pedido {$order['id']} já foi integrado pelo sistema.", 302);
        }
        
        $deliveryFee = 0;
                
        if(isset($order['extras']) && !empty($order['extras'])) {
            $benefits = $order['extras'];
            foreach($benefits as $benefit) {  // PASSAR ISSO LÁ PARA BAIXO!!!!!
                if($benefit['key'] == "additional" && $benefit['label'] == "Entrega"){
                    $deliveryFee = $benefit['value'];
                }
            }
        }
        
        $sale = $this->util->newCode("VENDAREST".$generalParams["CDFILIAL"], 10);
        $comd = $this->util->newCode("COMANDAVEN".$generalParams["CDFILIAL"], 10);
        
        $order["NRVENDAREST"] = $sale;
        $order["NRCOMANDA"]   = $comd;
        $order["IDSTCOMANDA"] = $generalParams["IDCONTROPROD"] === "S" ? "1" : "3";
        
        $this->orderFactory->insertSale($sale, $generalParams, $order, $consumer);

        $outsourcedDelivery = $deliveryRules['outsourcedDelivery'];

        $this->orderFactory->insertSaleCommand($generalParams, $order, $consumer, $deliveryFee);

        $total = $order["total_value"];
        $moneyCode = $this->paymentFactory->getMoneyReceiptCode();
        
        $discountOnyo = 0.0;

        if(isset($order['extras']) && !empty($order['extras'])) {
            $benefits = $order['extras'];
            foreach($benefits as $benefit) {
                if($benefit['key'] == "coupon"){
                    $intDiscountOnyo     = intval($benefit['discount_onyo']);
                    $couponValue         = -($benefit["value"]);
                    $coupomDiscount      = $couponValue;
                    if($intDiscountOnyo >= 1 && $intDiscountOnyo <= 99){
                        $coupomDiscount  = bcdiv(bcmul($coupomDiscount, $intDiscountOnyo, 2), 100, 2);
                        $coupomDiscount  = floatval($coupomDiscount);
                    }
                    
                    $this->processPayment($order, $coupomDiscount, $generalParams, $consumer, "cupom_pedepronto", $sale, null);
                    $order["discount_value"] = bcsub($order["discount_value"], $coupomDiscount, 2);
                }
            }
        }
        
        foreach ($order["payments"] as &$payment) {

            /*
             * Cupons de desconto
             * 
             * Sobrescreve a variavel discount_value com o valor do desconto informado. 
             * Ignora movimentação de caixa (MOVCAIXADLV) pois o valor pago considera o desconto aplicado.
             * Ao utilizar a variavel discount_value o desconto será rateado entre os demais itens do pedido (com valor superior a 1 centavo).
             * 
             */
             
            if(!preg_match('/^(?!^\s+$)[a-zA-Z0-9!@#$%^&*()\-_=+;:,.<>? \s]+$/u', $payment)){
                throw new Exception\ParamsException("
                        A integração recebeu um pedido com código de pagamento vazio.<br>
                        Entre em contato com o suporte da Pede Pronto para obter ajuda.");
            }
             
            if($payment['payment_method'] == "coupon") {
                $order["discount_value"] = bcadd($order["discount_value"], $payment['value'], 2);
                continue;
            }
            
            
            if(isset($payment['acquirer_return']['nsu']) && !empty($payment['acquirer_return']['nsu'])){
                $nsu = $payment['acquirer_return']['nsu'];
            } else {
                $nsu = null;
            }
            
            $this->processPayment($order, $payment["value"], $generalParams, $consumer, $payment["payment_method_pos_reference"], $sale, $nsu);

        }
        if (isset($payment))
            unset($payment);

        $generalParams["NRSEQPRODCUP"] = $this->util->newCode("ITCOMANDAVEN".$generalParams["CDFILIAL"], 10);

        $products     = [];
        $nrSeqProdCom = 0;
        
        $ITCOMANDAVEN_SEQ = 0;

        foreach ($order["nested_items"] as $item) {
            
            if(empty($item['pos_reference']) && ($item['unit_pos_price'] != null && $item['unit_pos_price'] != "0.00")){
                throw new Exception\ParamsException("
                        O produto {$item['pos_name']} possui preço e não possui código externo.<br>
                        Para realizar a correção, entre em contato com a equipe de suporte da Pede Pronto.");
            }
            
            if(!empty($item['pos_reference']) && ($item['unit_pos_price'] == null || $item['unit_pos_price'] == "0.00")) {
                    throw new Exception\ParamsException("
                        O produto {$item['pos_name']} possui preço inferior a R$ 0,01 centavo.<br>
                        Código do produto: {$item['pos_reference']}.<br>
                        Para realizar a correção, entre em contato com a equipe de suporte da Pede Pronto.");
            }
            
            $processedProduct = $this->productService->processProduct($generalParams, $item, $order, $nrSeqProdCom, $ITCOMANDAVEN_SEQ);
            
            if (is_array($processedProduct) && !isset($processedProduct["CDPRODUTO"])) {
                $products = array_merge($products, $processedProduct);
            } else {
                array_push($products, $processedProduct);
            }
        }
        
        if(empty($products)) {
            throw new Exception\ParamsException("
                Ocorreu uma falha na validação de produtos.<br>
                Contate a equipe de suporte da Teknisa.", 187);
        }
        
        if(bccomp($order["discount_value"], "0.00", 2) == 1) {
            $products = $this->productService->shareDiscount($order["discount_value"], $products);
        }

        foreach ($products as $product) {
            
            $valorProduto  = number_format($product['VRPRECCOMVEN'], 2, ',', '');
            $valorDesconto = number_format($product['VRDESCCOMVEN'], 2, ',', '');
            if(empty($product['VRPRECCOMVEN']) || $product['VRPRECCOMVEN'] < 0) {
                throw new Exception\ParamsException("
                    O produto {$product['NMPRODUTO']} possui preço inferior a R$ 0,01 centavo.<br>
                    Código do produto: {$product['CDPRODUTO']}.<br>
                    Valor do produto: R$ {$valorProduto}.<br>
                    Caso o produto seja um combo, em que o item pai não possui preço, é necessário excluir o código do produto do item pai.<br>
                    Para realizar a exclusão do código, entre em contato com a equipe de suporte da Pede Pronto.");
            }
            
            if($product['VRDESCCOMVEN'] >= $product['VRPRECCOMVEN']) {
                throw new Exception\ParamsException("
                    O produto {$product['NMPRODUTO']} possui desconto superior ao valor de venda.<br>
                    Valor do produto: R$ {$valorProduto}.<br>
                    Desconto aplicado: R$ {$valorDesconto}.<br>
                    Gentileza entrar em contato com a equipe de suporte da Teknisa.");
            }
            
            $this->productService->insertProduct(array_merge($product, $generalParams));
            
            //Observações da OBSITCOMANDAVEN
            if(!empty($product['ITEM_OBSERVATION'])) {
                foreach($product['ITEM_OBSERVATION'] as $itemObservation) {
                    $this->productService->insertObservation($itemObservation);
                }                
            }
            
        }
        
        return [
            'CDFILIAL'      => $generalParams["CDFILIAL"],
            'NRCOMANDA'     => $order["NRCOMANDA"],
            'NRCOMANDAEXT'  => $order["id"],
            'NRVENDAREST'   => $order["NRVENDAREST"]
        ];
    }
    
    public function deliveryRules($cdFilial) {
        $merchantConfiguration = $this->integrationFactory->getMerchantConfiguration($cdFilial);

        if(empty($merchantConfiguration)) {
            throw new Exception\ParamsException("
                Não existe um token atrelado a filial {$cdFilial}.<br>
                Para obter o token desta filial, entre em contato com o suporte da Pede Pronto.<br>
                Para inserir o token, verifique a seção 'Configurando o Token das Filiais', no manual da integração.");
        }

        return [
            'merchantConfiguration' => $merchantConfiguration,
            'outsourcedDelivery' => ($merchantConfiguration['IDUTLENTRETERC'] == "S" || $merchantConfiguration['IDRETTXENTR'] == "S")
        ];
    }
    
    public function validateOrder($cdFilial, $nrVendaRest, $orderId) {
        $this->orderFactory->validateOrder($cdFilial, $nrVendaRest, $orderId);
    }
    
    public function getConfiguration() {
        return $this->integrationFactory->getConfiguration();
    }
    
    public function processPayment($order, $value, $generalParams, $consumer, $paymentMethod, $sale, $nsu){
        $receiptType          = $this->paymentFactory->getReceiptType($paymentMethod);
        $order["NRSEQMOVDLV"] = $this->util->newCode("MOVCAIXADLV".$generalParams["CDFILIAL"].$sale, 10);
        $order["CDTIPORECE"]  = $receiptType;
        
        
        $this->orderFactory->insertCashMoviment($order, $value, $generalParams, $consumer,  $nsu);
        
    }
    
    public function disableIntegration() {
        $this->integrationFactory->disableIntegration();
    }
}
